--[[ Credit: Blazeflack]]

--Add access to ElvUI engine and unitframe framework
local E = unpack(ElvUI);
local ElvUF = ElvUI.oUF
assert(ElvUF, "ElvUI was unable to locate oUF.")

--Cache global variables
--Lua functions
local _G = _G
local unpack, pairs, assert = unpack, pairs, assert
local twipe = table.wipe
local ceil, sqrt, floor, abs = math.ceil, math.sqrt, math.floor, math.abs
local format, strupper, len = string.format, strupper, string.len
--WoW API
local GetNumGroupMembers = GetNumGroupMembers
local IsInGroup = IsInGroup
local IsInRaid = IsInRaid
local UnitClass = UnitClass
local UnitFactionGroup = UnitFactionGroup
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitIsDead = UnitIsDead
local UnitIsGhost = UnitIsGhost
local UnitIsUnit = UnitIsUnit
local UnitName = UnitName
local UnitPower = UnitPower
local UnitPowerMax = UnitPowerMax
local UnitPowerType = UnitPowerType

--GLOBALS: _TAGS, Hex, _COLORS

local textFormatStyles = {
	["CURRENT"] = "%s",
	["CURRENT_MAX"] = "%s - %s",
	["CURRENT_PERCENT"] =  "%s - %.1f%%",
	["CURRENT_MAX_PERCENT"] = "%s - %s | %.1f%%",
	["PERCENT"] = "%.1f%%",
	["DEFICIT"] = "-%s"
}

local textFormatStylesNoDecimal = {
	["CURRENT"] = "%s",
	["CURRENT_MAX"] = "%s - %s",
	["CURRENT_PERCENT"] =  "%s - %.0f%%",
	["CURRENT_MAX_PERCENT"] = "%s - %s | %.0f%%",
	["PERCENT"] = "%.0f%%",
	["DEFICIT"] = "-%s"
}

local shortValueFormat
local function ShortValue(number, noDecimal)
	shortValueFormat = (noDecimal and "%.0f%s" or format("%%.%df", E.db.general.decimalLength or 1))
	if E.db.general.numberPrefixStyle == "METRIC" then
		if abs(number) >= 1e12 then
			return format("%.1f%s", number / 1e12, "T")
		elseif abs(number) >= 1e9 then
			return format("%.1f%s", number / 1e9, "G")
		elseif abs(number) >= 1e6 then
			return format("%.1f%s", number / 1e6, "M")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "k")
		else
			return format("%.0f", number)
		end
	elseif E.db.general.numberPrefixStyle == "CHINESE" then
		if abs(number) >= 1e8 then
			return format("%.1f%s", number / 1e8, "Y")
		elseif abs(number) >= 1e4 then
			return format("%.1f%s", number / 1e4, "W")
		else
			return format("%.0f", number)
		end
	elseif E.db.general.numberPrefixStyle == "KOREAN" then
		if abs(number) >= 1e8 then
			return format("%.1f%s", number / 1e8, "억")
		elseif abs(number) >= 1e4 then
			return format("%.1f%s", number / 1e4, "만")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "천")
		else
			return format("%.0f", number)
		end
	elseif E.db.general.numberPrefixStyle == "GERMAN" then
		if abs(number) >= 1e12 then
			return format("%.1f%s", number / 1e12, "Bio")
		elseif abs(number) >= 1e9 then
			return format("%.1f%s", number / 1e9, "Mrd")
		elseif abs(number) >= 1e6 then
			return format("%.1f%s", number / 1e6, "Mio")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "Tsd")
		else
			return format("%.0f", number)
		end
	else
		if abs(number) >= 1e12 then
			return format("%.1f%s", number / 1e12, "T")
		elseif abs(number) >= 1e9 then
			return format("%.1f%s", number / 1e9, "B")
		elseif abs(number) >= 1e6 then
			return format("%.1f%s", number / 1e6, "M")
		elseif abs(number) >= 1e3 then
			return format(shortValueFormat, number / 1e3, "K")
		else
			return format("%.0f", number)
		end
	end
end

local function GetFormattedText(min, max, style, noDecimal)
	assert(textFormatStyles[style] or textFormatStylesNoDecimal[style], "CustomTags Invalid format style: "..style)
	assert(min, "CustomTags - You need to provide a current value. Usage: GetFormattedText(min, max, style, noDecimal)")
	assert(max, "CustomTags - You need to provide a maximum value. Usage: GetFormattedText(min, max, style, noDecimal)")

	if max == 0 then max = 1 end

	local chosenFormat
	if noDecimal then
		chosenFormat = textFormatStylesNoDecimal[style]
	else
		chosenFormat = textFormatStyles[style]
	end

	if style == "DEFICIT" then
		local deficit = max - min
		if deficit <= 0 then
			return nil
		else
			return format(chosenFormat, ShortValue(deficit, noDecimal))
		end
	elseif style == "PERCENT" then
		return format(chosenFormat, min / max * 100)
	elseif style == "CURRENT" or ((style == "CURRENT_MAX" or style == "CURRENT_MAX_PERCENT" or style == "CURRENT_PERCENT") and min == max) then
		if noDecimal then
			return format(textFormatStylesNoDecimal["CURRENT"], ShortValue(min, noDecimal))
		else
			return format(textFormatStyles["CURRENT"], ShortValue(min, noDecimal))
		end
	elseif style == "CURRENT_MAX" then
		return format(chosenFormat, ShortValue(min, noDecimal), ShortValue(max, noDecimal))
	elseif style == "CURRENT_PERCENT" then
		return format(chosenFormat, ShortValue(min, noDecimal), min / max * 100)
	elseif style == "CURRENT_MAX_PERCENT" then
		return format(chosenFormat, ShortValue(min, noDecimal), ShortValue(max, noDecimal), min / max * 100)
	end
end

--[[
	Custom Tags
--]]

-- [name:caps] - Displays the name of the unit in ALL CAPS.
ElvUF.Tags.Events['name:caps'] = 'UNIT_NAME_UPDATE'
ElvUF.Tags.Methods['name:caps'] = function(unit)
    local name = UnitName(unit)
    return name ~= nil and strupper(name) or ''
end

-- [name:abbreviate] - Abbreviates the name (Raid Target Dummy = R. T. Dummy).
ElvUF.Tags.Events['name:abbreviate'] = 'UNIT_NAME_UPDATE'
ElvUF.Tags.Methods['name:abbreviate'] = function(unit)
	local name = UnitName(unit)

	if name then
		name = name:gsub('(%S+) ', function(t) return t:sub(1,1)..'. ' end)
	end

	return name
end

-- [name:veryshort:abbreviate] - Abbreviates the name if it is longer than 5 characters.
ElvUF.Tags.Events['name:veryshort:abbreviate'] = 'UNIT_NAME_UPDATE'
ElvUF.Tags.Methods['name:veryshort:abbreviate'] = function(unit)
	local name = UnitName(unit)

	if name and len(name) > 5 then
		name = name:gsub('(%S+) ', function(t) return t:sub(1,1)..'. ' end)
	end

	return name
end

-- [name:short:abbreviate] - Abbreviates the name if it is longer than 10 characters.
ElvUF.Tags.Events['name:short:abbreviate'] = 'UNIT_NAME_UPDATE'
ElvUF.Tags.Methods['name:short:abbreviate'] = function(unit)
	local name = UnitName(unit)

	if name and len(name) > 10 then
		name = name:gsub('(%S+) ', function(t) return t:sub(1,1)..'. ' end)
	end

	return name
end

-- [name:medium:abbreviate] - Abbreviates the name if it is longer than 15 characters.
ElvUF.Tags.Events['name:medium:abbreviate'] = 'UNIT_NAME_UPDATE'
ElvUF.Tags.Methods['name:medium:abbreviate'] = function(unit)
	local name = UnitName(unit)

	if name and len(name) > 15 then
		name = name:gsub('(%S+) ', function(t) return t:sub(1,1)..'. ' end)
	end

	return name
end

-- [name:long:abbreviate] - Abbreviates the name if it is longer than 20 characters.
ElvUF.Tags.Events['name:long:abbreviate'] = 'UNIT_NAME_UPDATE'
ElvUF.Tags.Methods['name:long:abbreviate'] = function(unit)
	local name = UnitName(unit)

	if name and len(name) > 20 then
		name = name:gsub('(%S+) ', function(t) return t:sub(1,1)..'. ' end)
	end

	return name
end

-- [classcolor:player] - Colors the next tags with class color of the player.
ElvUF.Tags.Methods["classcolor:player"] = function()
	local _, unitClass = UnitClass("player")
	local String

	if unitClass then
		String = Hex(_COLORS.class[unitClass])
	else
		String = "|cFFC2C2C2"
	end
	
	return String
end

-- [classcolor:< class >] - Colors the next tags with class color of the supplied class. Example: [classcolor:warlock].
ElvUF.Tags.Methods["classcolor:hunter"] = function()
	return Hex(_COLORS.class["HUNTER"])
end

ElvUF.Tags.Methods["classcolor:warrior"] = function()
	return Hex(_COLORS.class["WARRIOR"])
end

ElvUF.Tags.Methods["classcolor:paladin"] = function()
	return Hex(_COLORS.class["PALADIN"])
end

ElvUF.Tags.Methods["classcolor:mage"] = function()
	return Hex(_COLORS.class["MAGE"])
end

ElvUF.Tags.Methods["classcolor:priest"] = function()
	return Hex(_COLORS.class["PRIEST"])
end

ElvUF.Tags.Methods["classcolor:warlock"] = function()
	return Hex(_COLORS.class["WARLOCK"])
end

ElvUF.Tags.Methods["classcolor:shaman"] = function()
	return Hex(_COLORS.class["SHAMAN"])
end

ElvUF.Tags.Methods["classcolor:deathknight"] = function()
	return Hex(_COLORS.class["DEATHKNIGHT"])
end

ElvUF.Tags.Methods["classcolor:druid"] = function()
	return Hex(_COLORS.class["DRUID"])
end

ElvUF.Tags.Methods["classcolor:monk"] = function()
	return Hex(_COLORS.class["MONK"])
end

ElvUF.Tags.Methods["classcolor:rogue"] = function()
	return Hex(_COLORS.class["ROGUE"])
end

ElvUF.Tags.Methods["classcolor:demonhunter"] = function()
	return Hex(_COLORS.class["DEMONHUNTER"])
end